/* BFDefaultInfoPanel.m created by lindberg on Wed 24-May-2000 */

#import "BFDefaultInfoPanel.h"

#ifdef OPENSTEP
#import <AppKit/AppKit.h>

@implementation BFDefaultInfoPanel

+ (id)allocWithZone:(NSZone *)aZone
{
    return [self sharedInstance];
}

+ (id)sharedInstance
{
    static id instance = nil;
    if (instance == nil) {
        instance = [[super allocWithZone:NULL] initWithWindowNibName:@"BFDefaultInfoPanel"];
        [instance setWindowFrameAutosaveName:@"InfoPanel"];
        [instance setShouldCascadeWindows:NO];
    }

    return instance;
}

- (void)setValuesFromOptions:(NSDictionary *)options
{
    NSBundle     *appBundle = [NSBundle mainBundle];
    NSDictionary *infoDict = [appBundle infoDictionary];
    NSString     *creditsFile;
    NSString     *name;
    NSString     *versionString;
    NSString     *build;
    NSString     *copyright;

    name = [options objectForKey:@"ApplicationName"];
    if (name == nil) name = [infoDict objectForKey:@"NSHumanReadableShortName"];
    if (name == nil) name = [[NSProcessInfo processInfo] processName];
    [appNameField setStringValue:name];

    if ([options objectForKey:@"ApplicationIcon"])
        [appIconView setImage:[options objectForKey:@"ApplicationIcon"]];

    versionString = [options objectForKey:@"ApplicationVersion"];
    build         = [options objectForKey:@"Version"];
    if (versionString == nil) versionString = [infoDict objectForKey:@"NSAppVersion"];
    if (versionString == nil) versionString = @"Version XX.x";
    if (build == nil) build = [infoDict objectForKey:@"NSBuildVersion"];
    if ([build length] > 0)
        versionString = [versionString stringByAppendingFormat:@" (v%@)", build];
    [versionField setStringValue:versionString];

    copyright = [options objectForKey:@"Copyright"];
    if (copyright == nil) copyright = [infoDict objectForKey:@"NSHumanReadableCopyright"];
    [legalTextField setStringValue:copyright];

    if ([options objectForKey:@"Credits"])
    {
        [[creditView textStorage] setAttributedString:[options objectForKey:@"Credits"]];
    }
    else
    {
        creditsFile = [appBundle pathForResource:@"Credits" ofType:@"rtfd"];
        if (creditsFile == nil) creditsFile = [appBundle pathForResource:@"Credits" ofType:@"rtf"];
        if (creditsFile) [creditView readRTFDFromFile:creditsFile];
    }
	
    [creditScrollView setHasVerticalScroller:
        NSHeight([creditView frame]) > NSHeight([creditScrollView frame])];
}

- (void)windowDidLoad
{
    [super windowDidLoad];
    [creditView setDrawsBackground:NO];
    [self setValuesFromOptions:nil];
}

- (void)showWindowWithOptions:(NSDictionary *)options
{
    if (![self isWindowLoaded]) {
        [self window]; // trigger load
    }

    if (options) [self setValuesFromOptions:options];
    [self showWindow:nil];
}

@end


@implementation NSApplication (BFStandardInfoPanel)

- (void)orderFrontStandardAboutPanel:(id)sender
{
    [self orderFrontStandardAboutPanelWithOptions:nil];
}

- (void)orderFrontStandardAboutPanelWithOptions:(NSDictionary *)optionsDictionary
{
    [[BFDefaultInfoPanel sharedInstance] showWindowWithOptions:optionsDictionary];
}

/* Optional keys in optionsDictionary:

@"Credits": NSAttributedString displayed in the info area of the panel. If
not specified, contents obtained from "Credits.rtf" in [NSBundle mainBundle];
if not available, blank.

@"ApplicationName": NSString displayed in place of the default app name. If
not specified, uses the value of NSHumanReadableShortName in the localized version
of Info.plist. If that's not available, uses [[NSProcessInfo processInfo]
processName].

@"ApplicationIcon": NSImage displayed in place of NSApplicationIcon. If not
specified, use [NSImage imageNamed:@"NSApplicationIcon"]; if not available,
generic icon.

@"Version": NSString containing the build version number of the application
("58.4"); displayed as "(v58.4)". If not specified, obtain from the
NSBuildVersion key in infoDictionary; if not specified, leave blank (the "(v)"
is not displayed).

@"Copyright": NSString containing the copyright string. If not specified,
obtain from the value of NSHumanReadableCopyright in the localized version
Info.plist; if not available, leave blank.

@"ApplicationVersion": NSString displayed as the application version
("Rhapsody 1.0", "WebObjects 3.5", "ClarisWorks 5", ...). If not specified,
obtain from the NSAppVersion key in infoDictionary. If not available, left
blank; version string will be displayed as "Version XX.x"

*/
@end

@interface _BFTransparentClipView : NSClipView
@end

@implementation _BFTransparentClipView
- (BOOL)isOpaque { return NO; }
- (void)drawRect:(NSRect)frame {}
@end

@interface _BFTransparentScrollView : NSScrollView
@end

@implementation _BFTransparentScrollView

- (void)awakeFromNib
{
    id newClipView = [[_BFTransparentClipView alloc] initWithFrame:[[self contentView] frame]];
    [newClipView setCopiesOnScroll:NO];
    [newClipView setDocumentView:[self documentView]];
    [self setContentView:newClipView];
    [newClipView release];
}

- (BOOL)isOpaque { return NO; }
- (void)drawRect:(NSRect)rect
{
    NSRectEdge side;
    float color = NSBlack;
    NSRect lineRect;

    if (_sFlags.needsTile) [self tile]; // making sure

    /* 
     * NSScrollView seems to draw lines between the scrollers and the clip view,
     * then draw the background color. We want the lines but not the background.
     */
    if ([self hasVerticalScroller])
    {
        lineRect = [[self verticalScroller] frame];
        lineRect.origin.x++;
        side = NSMaxXEdge;
        NSDrawTiledRects(lineRect, rect, &side, &color, 1);
    }
    if ([self hasHorizontalScroller])
    {
        lineRect = [[self horizontalScroller] frame];
        lineRect.origin.y++;
        side = NSMaxYEdge;
        NSDrawTiledRects(lineRect, rect, &side, &color, 1);
    }
}

@end

#endif
